<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       www.wbcomdesigns.com
 * @since      1.0.0
 *
 * @package    Woo_Sell_Services
 * @subpackage Woo_Sell_Services/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Woo_Sell_Services
 * @subpackage Woo_Sell_Services/admin
 * @author     Wbcom Designs <admin@wbcomdesigns.com>
 */
class Woo_Sell_Services_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * All tabs of settings page.
	 *
	 * @since    1.0.0
	 * @access   public
	 * @var      array    $plugin_settings_tabs  The tabs of plugin's admin settings.
	 */
	private $plugin_settings_tabs;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since 1.0.0
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version     The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		// $this->woo_sell_required_pages();
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function enqueue_styles() {
		$screen = get_current_screen();
		if ( 'wb-plugins_page_woo-sell-services' === $screen->base ) {
			wp_enqueue_style( 'wp-color-picker' );
			if ( ! wp_style_is( 'woo-sell-services-selectize-css', 'enqueued' ) ) {
				wp_enqueue_style( 'woo-sell-services-selectize-css', plugin_dir_url( __FILE__ ) . 'assets/css/selectize.css', array(), $this->version, 'all' );
			}
			if ( ! wp_style_is( 'woo-sell-services-admin-css', 'enqueued' ) ) {
				wp_enqueue_style( 'woo-sell-services-admin-css', plugin_dir_url( __FILE__ ) . 'assets/css/woo-sell-services-admin.css', array(), $this->version, 'all' );
			}
		}

		wp_register_style(
			'wbcom_wss_req_manager_css',
			WOO_SELL_PLUGIN_URL . 'admin/assets/css/requirement-manager.css',
			array(),
			time(),
			'all'
		);
		if ( ! wp_style_is( 'wbcom_wss_req_manager_css', 'enqueued' ) ) {
			wp_enqueue_style( 'wbcom_wss_req_manager_css' );
		}
	}

	     /**
		 * Hide all notices from the setting page.
		 *
		 * @return void
		 */
		public function wbcom_hide_all_admin_notices_from_setting_page() {
			$wbcom_pages_array  = array( 'wbcomplugins', 'wbcom-plugins-page', 'wbcom-support-page', 'woo-sell-services' );
			$wbcom_setting_page = filter_input( INPUT_GET, 'page' ) ? filter_input( INPUT_GET, 'page' ) : '';

			if ( in_array( $wbcom_setting_page, $wbcom_pages_array, true ) ) {
				remove_all_actions( 'admin_notices' );
				remove_all_actions( 'all_admin_notices' );
			}

		}

	/**
	 * Register the script for the admin area.
	 *
	 * @since    1.0.0
	 * @author   Wbcom Designs
	 * @access   public
	 */
	public function enqueue_scripts() {
		$screen = get_current_screen();
		if ( 'wb-plugins_page_woo-sell-services' === $screen->base ) {
			wp_enqueue_script( 'wp-color-picker');
			if ( ! wp_script_is( 'jquery', 'enqueued' ) ) {
				wp_enqueue_script( 'jquery' );
			}
			if ( ! wp_script_is( 'jquery-ui-sortable', 'enqueued' ) ) {
				wp_enqueue_script( 'jquery-ui-sortable' );
			}
			if ( ! wp_script_is( 'woo-sell-services-selectize-js', 'enqueued' ) ) {
				wp_enqueue_script( 'woo-sell-services-selectize-js', plugin_dir_url( __FILE__ ) . 'assets/js/selectize.min.js', array( 'jquery' ), $this->version, false );
			}
			if ( ! wp_script_is( 'woo-sell-services-admin-js', 'enqueued' ) ) {
				wp_enqueue_script( 'woo-sell-services-admin-js', plugin_dir_url( __FILE__ ) . 'assets/js/woo-sell-services-admin.js', array( 'jquery' ), $this->version, false );
				wp_localize_script( 'woo-sell-services-admin-js', 'woo_sell_ajax_url', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );
			}
		}

		wp_register_script(
			'wbcom_wss_req_manager',
			WOO_SELL_PLUGIN_URL . 'admin/assets/js/requirement-manager.js',
			array( 'jquery' ),
			time(),
			true
		);

		if ( ! wp_script_is( 'wbcom_wss_req_manager', 'enqueued' ) ) {
			wp_enqueue_script( 'wbcom_wss_req_manager' );
		}
	}

	/**
	 * Register a submenu in admin area ( In case of other wbcom plugin exists ) or a menu with submenu.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function wss_add_plugin_menu_page() {
		if ( empty( $GLOBALS['admin_page_hooks']['wbcomplugins'] ) ) {
			add_menu_page( esc_html__( 'WB Plugins', 'woo-sell-services' ), esc_html__( 'WB Plugins', 'woo-sell-services' ), 'manage_options', 'wbcomplugins', array( $this, 'wss_admin_settings_page' ), 'dashicons-lightbulb', 59 );
			add_submenu_page( 'wbcomplugins', esc_html__( 'General', 'woo-sell-services' ), esc_html__( 'General', 'woo-sell-services' ), 'manage_options', 'wbcomplugins' );
		}
		add_submenu_page( 'wbcomplugins', esc_html__( 'Woo Sell Services', 'woo-sell-services' ), esc_html__( 'Woo Sell Services', 'woo-sell-services' ), 'manage_options', 'woo-sell-services', array( $this, 'wss_admin_settings_page' ) );
	}

	/**
	 * Actions performed to display submenu page.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function wss_admin_settings_page() {
		$current = ( filter_input( INPUT_GET, 'tab' ) !== null ) ? filter_input( INPUT_GET, 'tab' ) : 'wbwss-welcome';
		?>
		<div class="wrap">
		<div class="wbcom-bb-plugins-offer-wrapper">
				<div id="wb_admin_logo">
					<a href="https://wbcomdesigns.com/downloads/buddypress-community-bundle/" target="_blank">
						<img src="<?php echo esc_url( WOO_SELL_PLUGIN_URL ) . 'admin/wbcom/assets/imgs/wbcom-offer-notice.png'; ?>">
					</a>
				</div>
			</div>
					<div class="wbcom-wrap">
					<div class="blpro-header">
					<div class="wbcom_admin_header-wrapper">
						<div id="wb_admin_plugin_name">
							<?php esc_html_e( 'Woo Sell Services', 'woo-sell-services' ); ?>
							<span><?php printf( __( 'Version %s', 'woo-sell-services' ), WOO_SELL_PLUGIN_VERSION ); ?></span>
						</div>
						<?php echo do_shortcode( '[wbcom_admin_setting_header]' ); ?>
					</div>
				</div>
			<?php settings_errors(); ?>
			<div class="wbcom-admin-settings-page">
				<?php
				$this->wbwss_plugin_settings_tabs();
				settings_fields( $current );
				do_settings_sections( $current );
				?>
			</div>
					</div>
		</div>
		<?php
	}

	/**
	 * Actions performed to create tabs on the sub menu page.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function wbwss_plugin_settings_tabs() {

		$current = ( filter_input( INPUT_GET, 'tab' ) !== null ) ? filter_input( INPUT_GET, 'tab' ) : 'wbwss-welcome';

		$tab_html = '<div class="wbcom-tabs-section"><div class="nav-tab-wrapper"><div class="wb-responsive-menu"><span>' . esc_html( 'Menu' ) . '</span><input class="wb-toggle-btn" type="checkbox" id="wb-toggle-btn"><label class="wb-toggle-icon" for="wb-toggle-btn"><span class="wb-icon-bars"></span></label></div><ul>';
		foreach ( $this->plugin_settings_tabs as $wss_tab => $wss_name ) {
			$class = ( $wss_tab === $current ) ? 'nav-tab-active' : '';
			$page  = 'woo-sell-services';
			if ( 'email' === $wss_tab ) {
				$page = 'wc-settings';
			}
			$tab_html .= '<li><a id="' . $wss_tab . '" class="nav-tab ' . $class . '" href="admin.php?page=' . $page . '&tab=' . $wss_tab . '">' . $wss_name . '</a></li>';
		}
		$tab_html .= '</div></ul></div>';
		echo wp_kses_post( $tab_html );
	}

	/**
	 * Register all settings.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function wss_add_plugin_register_settings() {
		$this->plugin_settings_tabs['wbwss-welcome'] = esc_html__( 'Welcome', 'woo-sell-services' );
		add_settings_section( 'wbwss-welcome', ' ', array( $this, 'wss_welcome_settings_content' ), 'wbwss-welcome' );

		$this->plugin_settings_tabs['wbwss-general'] = esc_html__( 'General', 'woo-sell-services' );
		register_setting( 'wbwss_general_settings', 'wbwss_general_settings' );
		add_settings_section( 'wbwss-general', ' ', array( $this, 'wss_general_settings_content' ), 'wbwss-general' );

		$this->plugin_settings_tabs['wbwss-requirement-images'] = esc_html__( 'Images', 'woo-sell-services' );
		register_setting( 'wbwss_requirement_images_settings', 'wbwss_requirement_images_settings', array( $this, 'wbwss_requirement_images_settings_callback' ) );
		add_settings_section( 'wbwss-requirement-images', ' ', array( $this, 'wbwss_requirement_images_settings_content' ), 'wbwss-requirement-images' );
		$this->plugin_settings_tabs['email'] = esc_html__( 'Emails', 'woo-sell-services' );

		$this->plugin_settings_tabs['wbwss-live-notification'] = esc_html__( 'Notification', 'woo-sell-services' );
		register_setting( 'wbwss_live_notification_settings', 'wbwss_live_notification_settings', array( $this, 'wbwss_live_notification_settings_callback' ) );
		add_settings_section( 'wbwss-requirement-notification', ' ', array( $this, 'wbwss_live_notification_settings_content' ), 'wbwss-live-notification' );
	}

	/**
	 * Welcome Tab Content.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function wss_welcome_settings_content() {
		require_once 'partials/wss-welcome-page.php';
	}

	/**
	 * General Tab Content.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function wss_general_settings_content() {
		require_once 'partials/wss-general-settings.php';
	}

	/**
	 * Images Tab Content.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function wbwss_requirement_images_settings_content() {
		require_once 'partials/wss-requirement-images-settings.php';
	}


	/**
	 * Notification Tab Content.
	 *
	 * @since  2.3.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function wbwss_live_notification_settings_content() {
		require_once 'partials/wss-requirement-notification-settings.php';
	}

	/**
	 * Callback function for plugin's general settings.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 * @param  array $input The uploaded image data.
	 */
	public function wbwss_requirement_images_settings_callback( $input ) {

		$wbwss_requirement_images_settings = get_option( 'wbwss_requirement_images_settings' );
		if ( isset( $_FILES ) ) {

			if ( ! function_exists( 'wp_handle_upload' ) ) {
				require_once ABSPATH . 'wp-admin/includes/file.php';
			}
			$upload_overrides = array( 'test_form' => false );
			foreach ( $_FILES as $data => $file_data_info ) {
				foreach ( $file_data_info['name'] as $image_name => $file_info ) {
					$uploadedfile = array(
						'name'     => $file_data_info['name'][ $image_name ],
						'type'     => $file_data_info['type'][ $image_name ],
						'tmp_name' => $file_data_info['tmp_name'][ $image_name ],
						'error'    => $file_data_info['error'][ $image_name ],
						'size'     => $file_data_info['size'][ $image_name ],
					);

					$movefile = wp_handle_upload( $uploadedfile, $upload_overrides );
					if ( $movefile && ! isset( $movefile['error'] ) ) {
						$input[ $image_name ] = $movefile['url'];
					} else {
						$input[ $image_name ] = isset( $wbwss_requirement_images_settings[ $image_name ] ) ? $wbwss_requirement_images_settings[ $image_name ] : '';
						echo esc_html( $movefile['error'] );
					}
				}
			}
		}

		return $input;
	}


	/**
	 * Callback function for plugin's general settings.
	 *
	 * @since  2.3.0
	 * @author Wbcom Designs
	 * @access public
	 * @param  array $input The uploaded Notification data.
	 */
	public function wbwss_live_notification_settings_callback( $input ) {

		return $input;
	}
	/**
	 * Update requirement images option if option field is empty.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function wss_update_default_requirement_images() {
		$default_images = array(
			'order_started'     => WOO_SELL_PLUGIN_URL . 'public/assets/images/order-started.png',
			'order_rqrmnt'      => WOO_SELL_PLUGIN_URL . 'public/assets/images/order-requirements.png',
			'order_conversation'=> WOO_SELL_PLUGIN_URL . 'public/assets/images/order-conversation.png',
			'final_dlvry_sent'  => WOO_SELL_PLUGIN_URL . 'public/assets/images/order-delivery.png',
			'final_dlvry_accpt' => WOO_SELL_PLUGIN_URL . 'public/assets/images/order-delivery-accept.png',
			'final_dlvry_deny'  => WOO_SELL_PLUGIN_URL . 'public/assets/images/order-delivery-deny.png',
			'order_review'      => WOO_SELL_PLUGIN_URL . 'public/assets/images/order-review.png',
			'order_comp'        => WOO_SELL_PLUGIN_URL . 'public/assets/images/order-complete.png',
		);

		$wbwss_requirement_images_settings = get_option( 'wbwss_requirement_images_settings' );
		if ( ! $wbwss_requirement_images_settings ) {
			update_option( 'wbwss_requirement_images_settings', $default_images );
		}
	}

	/**
	 * Register order single page.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function woo_sell_required_pages() {
		$wbwss_general_settings = get_option( 'wbwss_general_settings', true );
		/* woo sell service order page assign then will not create new order details page*/
		if ( isset( $wbwss_general_settings['woo_sell_service_order_page'] ) && '' != $wbwss_general_settings['woo_sell_service_order_page'] ) {
			return;
		}

		$required_page = get_posts(
			array(
				'pagename'  => 'service',
				'post_type' => 'page',
			)
		);
		$page_id       = '';
		if ( ! empty( $required_page ) && is_array( $required_page ) ) {
			foreach ( $required_page as $pages ) {
				$page_id = $pages->ID;
			}
		}
		if ( empty( $required_page ) ) {

			if ( empty( $wbwss_general_settings ) || '' == $wbwss_general_settings || ! is_array( $wbwss_general_settings ) ) {

				$wbwss_general_settings                                      = array();
				$wbwss_general_settings['enable_service_email_settings']     = 'on';
				$wbwss_general_settings['enable_conversation_email_setting'] = 'on';
				$wbwss_general_settings['conversation_notification']         = 'on';
				$wbwss_general_settings['enable_conversation_notification_tab']   = 'on';
				$wbwss_general_settings['enable_review_sync_woo_reviews']         = 'on';
				$wbwss_general_settings['enable_redirect_conversation_page']      = 'on';
				$wbwss_general_settings['enable_email_support_agent']             = 'off';
				$wbwss_general_settings['enable_live_conversation']               = 'off';
				$wbwss_general_settings['enable_conversation_coauthor']           = 'off';
				$wbwss_general_settings['notice_payment_hold']                    = 'You order has been [order-status]. Please contact to site administrator';
				$wbwss_general_settings['notice_payment_canceled']                = 'You order has been [order-status]. Please contact to site administrator';
				$wbwss_general_settings['wss_seller_user_roles']                  = array( 'author', 'shop_manager', 'administrator', 'vendor','seller','wcfm_vendor' );
				$wbwss_general_settings['wss_customer_notification']              = '12';
				$wbwss_general_settings['woo_sell_service_order_page']            = '';
				$wbwss_general_settings['wss_auto_complete_after_final_delivery'] = '5';
			}
			$woo_page    = array(
				'post_type'    => 'page',
				'post_name'    => 'service',
				'post_title'   => esc_html__( 'Order Details', 'woo-sell-services' ),
				'post_content' => '[wss_single_order]',
				'post_status'  => 'publish',
			);
			$woo_page_id = wp_insert_post( $woo_page );
			update_post_meta( $woo_page_id, WOO_SELL_SERVICES_PAGES, 'created' );

			$wbwss_general_settings['woo_sell_service_order_page'] = $woo_page_id;
			update_option( 'wbwss_general_settings', $wbwss_general_settings );

		} else {
			$meta = get_post_meta( $page_id, WOO_SELL_SERVICES_PAGES, true );
			if ( empty( $meta ) ) {
				global $order_detail_errors;
				$order_detail_errors = esc_html__( "The 'Woo Sell Services' plugin requires 'Order Details' page with 'services' slug and this slug already exists.", 'woo-sell-services' );
				add_action( 'admin_notices', array( $this, 'woo_sell_error_notices' ) );
			}
		}
	}

	/**
	 * Create a table wss_sell_messages for store all conversation.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function woo_sell_conversation_table() {
		global $wpdb;
		if ( is_multisite() && is_plugin_active_for_network( plugin_basename( __FILE__ ) ) ) {
			$woo_sell_option = get_site_option( 'woo_sell_services_table' );
		} else {
			$woo_sell_option = get_option( 'woo_sell_services_table' );
		}
		$woo_table_name = $wpdb->prefix . 'wss_sell_messages';
		if ( $woo_table_name !== $wpdb->get_var( "show tables like '$woo_table_name'" ) ) {
			if ( is_multisite() && is_plugin_active_for_network( plugin_basename( __FILE__ ) ) ) {
				delete_site_option( 'woo_sell_services_table' );
			} else {
				delete_option( 'woo_sell_services_table' );
			}

			if ( empty( $woo_sell_option ) ) {
				$woo_table = $wpdb->get_charset_collate();
				$woo_sql   = "CREATE TABLE $woo_table_name (
						id mediumint(9) NOT NULL AUTO_INCREMENT PRIMARY KEY,
						author_id mediumint(9) NOT NULL,
						user_id mediumint(9) NOT NULL,
						order_id mediumint(9) NOT NULL,
						item_id mediumint(9) NOT NULL,
						product_id mediumint(9) NOT NULL,
						message text NOT NULL,
						msg_time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
						attachment text NOT NULL,
						msg_read ENUM('0','1') NOT NULL,
						UNIQUE KEY id (id)
					) $woo_table;";
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';
				dbDelta( $woo_sql );
				if ( is_multisite() && is_plugin_active_for_network( plugin_basename( __FILE__ ) ) ) {
					update_site_option( 'woo_sell_services_table', WOO_SELL_SERVICES_TABLE );
				} else {
					update_option( 'woo_sell_services_table', WOO_SELL_SERVICES_TABLE );
				}
			}
		} else {
			if ( empty( $woo_sell_option ) ) {
				global $wss_sell_messages;
				$wss_sell_messages = esc_html__( "The 'Woo Sell Services' plugin requires 'wss_sell_messages' table, and this table already exists.", 'woo-sell-services' );
				add_action( 'admin_notices', array( $this, 'woo_sell_service_messages_notices' ) );
			}
		}
	}

	/**
	 * Create a table wss_service_notifications for store all conversation.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function woo_sell_notification_table() {
		global $wpdb;
		if ( is_multisite() && is_plugin_active_for_network( plugin_basename( __FILE__ ) ) ) {
			$woo_sell_option = get_site_option( 'wss_service_notifications_table' );
		} else {
			$woo_sell_option = get_option( 'wss_service_notifications_table' );
		}
		$woo_table_name = $wpdb->prefix . 'wss_service_notifications';
		if ( $wpdb->get_var( "show tables like '$woo_table_name'" ) !== $woo_table_name ) {
			if ( is_multisite() && is_plugin_active_for_network( plugin_basename( __FILE__ ) ) ) {
				delete_site_option( 'wss_service_notifications_table' );
			} else {
				delete_option( 'wss_service_notifications_table' );
			}
			if ( empty( $woo_sell_option ) ) {
				$woo_table = $wpdb->get_charset_collate();
				$woo_sql   = "CREATE TABLE $woo_table_name (
						notification_id mediumint(9) NOT NULL AUTO_INCREMENT PRIMARY KEY UNIQUE KEY,
						vendor_id mediumint(9) NOT NULL,
						user_id mediumint(9) NOT NULL,
						order_id mediumint(9) NOT NULL,
						item_id mediumint(9) NOT NULL,
						product_id mediumint(9) NOT NULL,
						vendor_notifi mediumint(9) NOT NULL,
						customer_notifi mediumint(9) NOT NULL,
						vendor_read mediumint(9) NOT NULL,
						customer_read mediumint(9) NOT NULL,
						created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
						action varchar(100) NOT NULL
					) $woo_table;";
				require_once ABSPATH . 'wp-admin/includes/upgrade.php';
				dbDelta( $woo_sql );
				if ( is_multisite() && is_plugin_active_for_network( plugin_basename( __FILE__ ) ) ) {
					update_site_option( 'wss_service_notifications_table', WOO_SELL_NOTIFICATIONS_TABLE );
				} else {
					update_option( 'wss_service_notifications_table', WOO_SELL_NOTIFICATIONS_TABLE );
				}
			}
		} else {
			if ( empty( $woo_sell_option ) ) {
				global $wss_service_notifications;
				$wss_service_notifications = esc_html__( "The 'Woo Sell Services' plugin requires 'wss_service_notifications' table and this table already exists.", 'woo-sell-services' );
				add_action( 'admin_notices', array( $this, 'woo_sell_service_notificcation_notices' ) );
			}
		}
	}

	/**
	 * Display admin notces if table or pages exists.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function woo_sell_error_notices() {
		global $order_detail_errors;
		?>
		<div class="error notice is-dismissible">
			<p><?php echo esc_html( $order_detail_errors ); ?></p>
		</div>
		<?php
	}

	/**
	 * Display admin notces if service exists.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function woo_sell_service_notificcation_notices() {
		global $wss_service_notifications;
		?>
		<div class="error notice is-dismissible">
			<p><?php echo esc_html( $wss_service_notifications ); ?></p>
		</div>
		<?php
	}

	/**
	 * Display admin notces if coversation message enabled.
	 *
	 * @since  1.0.0
	 * @author Wbcom Designs
	 * @access public
	 */
	public function woo_sell_service_messages_notices() {
		global $wss_sell_messages;
		?>
		<div class="error notice is-dismissible">
			<p><?php echo esc_html( $wss_sell_messages ); ?></p>
		</div>
		<?php
	}

	/**
	 * Add new inputs to each variation
	 *
	 *  @param int     $loop          Position in the loop.
	 * @param array   $variation_data Variation data.
	 * @param WP_Post $variation      Post data.
	 */
	public function wss_service_product_after_variable_attributes( $loop, $variation_data, $variation ) {

		$product_id                       = $variation->post_parent;
		$is_wss_type                      = get_post_meta( $product_id, '_wss_type', true );
		$variable_wbcom_wss_estimate_time = get_post_meta( $variation->ID, 'variable_wbcom_wss_estimate_time', true );
		?>
		<div class="show_if_variation_service" 
		<?php
		if ( 'yes' != $is_wss_type ) :
			?>
		style="display: none;" <?php endif; ?>>
		<?php
			woocommerce_wp_text_input(
				array(
					'id'                => "variable_wbcom_wss_estimate_time{$loop}",
					'class'             => 'dokan-form-control',
					'name'              => "variable_wbcom_wss_estimate_time[{$loop}]",
					'value'             => $variable_wbcom_wss_estimate_time,
					'label'             => __( 'Estimated time in days', 'woo-sell-services' ),
					'description'       => __( 'Enter the estimated days when will deliver service.', 'woo-sell-services' ),
					'type'              => 'number',
					'desc_tip'          => true,
					'custom_attributes' => array(
						'step' => '1',
						'min'  => '0',
					),
				)
			);
		?>
		</div>
		<?php
	}

	/**
	 * Add new inputs to each variation of WC Vendors.
	 *
	 * @param int     $loop          Position in the loop.
	 * @param int     $variation_id  Variation ID.
	 * @param array   $variation_data Variation data.
	 * @param WP_Post $variation      Post data.
	 */
	public function wss_service_wcv_product_after_variable_attributes( $loop, $variation_id, $variation_data, $variation ) {
		$product_id                       = $variation->post_parent;
		$is_wss_type                      = get_post_meta( $product_id, '_wss_type', true );
		$variable_wbcom_wss_estimate_time = get_post_meta( $variation->ID, 'variable_wbcom_wss_estimate_time', true );
		?>
		<div class="show_if_variation_service" 
		<?php
		if ( 'yes' != $is_wss_type ) :
			?>
		style="display: none;" <?php endif; ?>>
		<?php
			woocommerce_wp_text_input(
				array(
					'id'                => "variable_wbcom_wss_estimate_time{$loop}",
					'class'             => 'dokan-form-control',
					'name'              => "variable_wbcom_wss_estimate_time[{$loop}]",
					'value'             => $variable_wbcom_wss_estimate_time,
					'label'             => __( 'Estimated time in days', 'woo-sell-services' ),
					'description'       => __( 'Enter the estimated days when will deliver service.', 'woo-sell-services' ),
					'type'              => 'number',
					'desc_tip'          => true,
					'custom_attributes' => array(
						'step' => '1',
						'min'  => '0',
					),
				)
			);
		?>
		</div>
		<?php

	}

	/**
	 * Set variation to  save.
	 *
	 * @param int $variation_id  Variation ID.
	 * @param int $i Index.
	 */
	public function wss_service_save_product_variation( $variation_id, $i ) {

		if ( isset( $_POST['variable_wbcom_wss_estimate_time'] ) ) {
			update_post_meta( $variation_id, 'variable_wbcom_wss_estimate_time', $_POST['variable_wbcom_wss_estimate_time'][ $i ] );
		}
	}

	/**
	 * WooCommerce Order Data.
	 *
	 * @param WC_Order $order Order.
	 */
	public function wss_service_admin_order_wise_coauthor( $order ) {
		if ( ! is_admin() ) {
			return;
		}

		$order_id                = $order->get_id();
		if ( woo_sell_service_is_hpos() ) {
			$order		= wc_get_order($order_id);
			$wss_order_service_users = $order->get_meta('wss_order_service_users');		
		} else {
			$wss_order_service_users = get_post_meta( $order_id, 'wss_order_service_users', true );
		}


		?>
		<p class="wss-order-co-author form-field form-field-wide wc-co-user"> 
			<label><?php esc_html_e( 'Assign additional vendor/support agent/co-author', 'woo-sell-services' ); ?></label>
			<?php
			$woo_sell_user_list = get_option( 'wbwss_general_settings' );
			$user_lists         = get_users( array( 'role__in' => $woo_sell_user_list['wss_seller_user_roles'] ) );

			?>
			<select name="wss_order_service_users" class="" >
				<option value=""><?php esc_html_e( 'Assign service order additional vendor/support agent/co-author', 'woo-sell-services' ); ?></option>
				<?php
				foreach ( $user_lists as $user_list ) {
					$roles                              = $user_list->roles['0'];
					$field['choices'][ $user_list->ID ] = $user_list->data->display_name;
					?>
					<option value="<?php echo esc_attr( $user_list->ID ); ?>" <?php selected( $wss_order_service_users, $user_list->ID ); ?>><?php echo esc_html( $user_list->data->display_name ); ?></option>
					<?php
				}
				?>
			</select>
		</p>
		<?php
	}

	/**
	 * Saved the Order Posts.
	 *
	 * @param int     $post_id Posts ID or Product ID.
	 * @param WP_Post $post The post object for the current post.
	 */
	public function wss_service_admin_order_save_post_shop_order( $post_id, $post ) {
		if ( isset( $_POST['wss_order_service_users'] ) ) {			
			update_post_meta( $post_id, 'wss_order_service_users', $_POST['wss_order_service_users'] );
		}
	}
	
	public function wss_service_admin_order_item_object_save( $order, $data_store ) {
		if ( isset( $_POST['wss_order_service_users'] ) ) {
			$order_id 		= $order->get_data()['order_id'];
			$old_order 		= wc_get_order( $order_id );
			$wss_order_service_users = $old_order->get_meta('wss_order_service_users');	
			
			if ( $wss_order_service_users != $_POST['wss_order_service_users'] ) {
				$old_order->update_meta_data( 'wss_order_service_users', $_POST['wss_order_service_users'] );
				$old_order->save();
			}
			update_post_meta( $order_id, 'wss_order_service_users', $_POST['wss_order_service_users'] );			
		}
	}
	
	
	public function wss_update_requirement_image() {
		$wss_req_image_key = $_POST['wss_req_image_key'];
		$wss_req_image = $_POST['wss_req_image'];
		$wbwss_requirement_images_settings = get_option( 'wbwss_requirement_images_settings' );
		
		$wbwss_requirement_images_settings[$wss_req_image_key] = $wss_req_image;

		update_option( 'wbwss_requirement_images_settings', $wbwss_requirement_images_settings );
		wp_die();
	}
	
	/**
	 * Display Export template button on ACF render field.
	 *
	 * @param array   $fields ACF fields.	 
	 * @param int     $post_id Posts ID or Product ID.	 
	 *
	 & @since 4.9.0
	 */
	public function woo_sell_service_render_fields_acf_template_export( $fields, $post_id ) {
		
		if ( isset($fields[0]) && isset($fields[0]['key']) && $fields[0]['key'] == 'field_5d888eee0e63d') {
			$wbcom_wss_req_ques = get_field( 'woo_sell_services_requirement_data', $post_id );
			?>
			<div class="acf-field wss-acf-field-section">
				<?php
				if ( !empty( $wbcom_wss_req_ques )) {
					?>
						<button id="wss_export_service_template" class="acf-button button button-primary" data-id="<?php echo esc_attr($post_id)?>"><?php esc_html_e('Save as requirement template?', 'woo-sell-services');?></button>
					<?php
				}
				
				$user_id = get_current_user_id();
				$saved_service_data = get_user_meta( $user_id, '_wss_saved_service_data', true);
				if ( isset($saved_service_data[$post_id])) {
					unset($saved_service_data[$post_id]);
				}
				
				if ( !empty($saved_service_data)) :
				?>
					<select id="_wss_import_saved_service_data">	
						<option value=""><?php esc_html_e( 'Select requirement template', 'woo-sell-services');?></option>
						<?php foreach($saved_service_data as $s_key=>$data):?>
							<option value="<?php echo esc_attr($s_key)?>"><?php echo esc_html(get_the_title($s_key) .' - Requirements');?></option>
						<?php endforeach;?>
					</select>
					<button id="wss_import_service_template" class="acf-button button button-primary" data-id="<?php echo esc_attr($post_id)?>" disabled><?php esc_html_e('Import requirement template', 'woo-sell-services');?></button>
				<?php
				endif;
				
				?>
				<p class="wss-success-msg" style="display:none"><?php esc_html_e( 'Successfull 
				saved requirement template.', 'woo-sell-services');?></p>
				<p class="wss-success-import-msg" style="display:none"><?php esc_html_e( 'Successfull import requirement template.', 'woo-sell-services');?></p>
			</div>
			<?php
		}
	}
	
	public function wss_export_service_data() {
		$user_id = get_current_user_id();
		$post_id = $_POST['post_id'];
		$wbcom_wss_req_ques = get_field( 'woo_sell_services_requirement_data', $post_id );
		
		$saved_service_data = get_user_meta( $user_id, '_wss_saved_service_data', true);		
		if ( empty($saved_service_data)) {
			$saved_service_data = array();
		}
		$saved_service_data[$post_id] = $wbcom_wss_req_ques;		
		update_user_meta($user_id, '_wss_saved_service_data', $saved_service_data);
			
			
		wp_die();
	}
	
	public function wss_import_service_data() {
		$user_id = get_current_user_id();
		$post_id = $_POST['post_id'];
		$import_post_id = $_POST['import_post_id'];
		$wbcom_wss_req_ques = get_field( 'woo_sell_services_requirement_data', $post_id );
		
		/*
		 * Delete Existing Requirements Data
		 */
		if ( !empty($wbcom_wss_req_ques)) {			
			foreach( $wbcom_wss_req_ques as $key=>$row_data) {				
				delete_row('woo_sell_services_requirement_data', 1, $post_id);
			}
		}
		$saved_service_data = get_user_meta( $user_id, '_wss_saved_service_data', true);
		$field = acf_maybe_get_field( 'woo_sell_services_requirement_data', $import_post_id );
		$wbcom_wss_req_ques = get_field( 'woo_sell_services_requirement_data', $import_post_id );		
		acf_update_value( $wbcom_wss_req_ques, $post_id, $field );
		
		wp_die();
	}

}
